@@ -12,8 +12,9 @@ from deploy_tools.cli import (
     keystore_option,
     nonce_option,
     retrieve_private_key,
+    validate_address,
 )
-from deploy_tools.deploy import build_transaction_options
+from deploy_tools.transact import build_transaction_options
 from eth_utils import is_checksum_address, to_checksum_address
 
 import pendulum
@@ -22,7 +23,17 @@ from tldeploy.identity import (
     deploy_identity_proxy_factory,
 )
 
-from .core import deploy_exchange, deploy_network, deploy_networks, deploy_unw_eth
+from .core import (
+    deploy_exchange,
+    deploy_network,
+    deploy_networks,
+    deploy_unw_eth,
+    migrate_networks,
+    verify_networks_migrations,
+    deploy_beacon,
+    deploy_and_migrate_networks_from_file,
+    NetworkSettings,
+)
 
 
 def report_version():
@@ -56,9 +67,10 @@ def cli(ctx, version):
 
 currency_network_contract_name_option = click.option(
     "--currency-network-contract-name",
-    help="name of the currency network contract to deploy (only use this for testing)",
+    help="name of the currency network contract to deploy, "
+    "(e.g. CurrencyNetworkOwnable, CurrencyNetwork, or  TestCurrencyNetwork)",
     default="CurrencyNetwork",
-    hidden=True,
+    hidden=False,
 )
 
 
@@ -192,18 +204,22 @@ def currencynetwork(
         gas=gas, gas_price=gas_price, nonce=nonce
     )
 
-    contract = deploy_network(
-        web3,
-        name,
-        symbol,
-        decimals,
+    network_settings = NetworkSettings(
+        name=name,
+        symbol=symbol,
+        decimals=decimals,
         fee_divisor=fee_divisor,
         default_interest_rate=default_interest_rate,
         custom_interests=custom_interests,
         prevent_mediator_interests=prevent_mediator_interests,
+        expiration_time=expiration_time,
+    )
+
+    contract = deploy_network(
+        web3,
+        network_settings,
         exchange_address=exchange_contract,
         currency_network_contract_name=currency_network_contract_name,
-        expiration_time=expiration_time,
         transaction_options=transaction_options,
         private_key=private_key,
     )
@@ -239,7 +255,7 @@ def exchange(
     jsonrpc: str, gas: int, gas_price: int, nonce: int, auto_nonce: bool, keystore: str
 ):
     """Deploy an exchange contract and a contract to wrap Ether into an ERC 20
-  token.
+    token.
     """
     web3 = connect_to_json_rpc(jsonrpc)
     private_key = retrieve_private_key(keystore)
@@ -305,8 +321,7 @@ def identity_implementation(
 def identity_proxy_factory(
     jsonrpc: str, gas: int, gas_price: int, nonce: int, auto_nonce: bool, keystore: str
 ):
-    """Deploy an identity proxy factory, which can be used to create proxies for identity contracts.
-    """
+    """Deploy an identity proxy factory, which can be used to create proxies for identity contracts."""
 
     web3 = connect_to_json_rpc(jsonrpc)
     private_key = retrieve_private_key(keystore)
@@ -351,38 +366,42 @@ def test(
     currency_network_contract_name: str,
 ):
     """Deploy three test currency network contracts connected to an exchange contract and an unwrapping ether contract.
-    Also deploys an identity proxy factory and a identity implementation contract.
+    Also deploys an identity proxy factory and an identity implementation contract.
     This can be used for testing"""
 
     expiration_time = 4_102_444_800  # 01/01/2100
 
     network_settings = [
-        {
-            "name": "Cash",
-            "symbol": "CASH",
-            "decimals": 4,
-            "fee_divisor": 1000,
-            "default_interest_rate": 0,
-            "custom_interests": True,
-            "expiration_time": expiration_time,
-        },
-        {
-            "name": "Work Hours",
-            "symbol": "HOU",
-            "decimals": 4,
-            "fee_divisor": 0,
-            "default_interest_rate": 1000,
-            "custom_interests": False,
-            "expiration_time": expiration_time,
-        },
-        {
-            "name": "Beers",
-            "symbol": "BEER",
-            "decimals": 0,
-            "fee_divisor": 0,
-            "custom_interests": False,
-            "expiration_time": expiration_time,
-        },
+        NetworkSettings(
+            name="Cash",
+            symbol="CASH",
+            decimals=4,
+            fee_divisor=1000,
+            default_interest_rate=0,
+            custom_interests=True,
+            expiration_time=expiration_time,
+            prevent_mediator_interests=False,
+        ),
+        NetworkSettings(
+            name="Work Hours",
+            symbol="HOU",
+            decimals=4,
+            fee_divisor=0,
+            default_interest_rate=1000,
+            custom_interests=False,
+            expiration_time=expiration_time,
+            prevent_mediator_interests=False,
+        ),
+        NetworkSettings(
+            name="Beers",
+            symbol="BEER",
+            decimals=0,
+            fee_divisor=0,
+            default_interest_rate=0,
+            custom_interests=False,
+            expiration_time=expiration_time,
+            prevent_mediator_interests=False,
+        ),
     ]
 
     web3 = connect_to_json_rpc(jsonrpc)
@@ -445,3 +464,195 @@ def test(
                 settings=settings, address=to_checksum_address(address)
             )
         )
+
+
+@cli.command(short_help="Migrate old currency networks to new ones.")
+@click.option(
+    "--old-addresses",
+    "old_addresses_file_path",
+    help="Path to a csv file with addresses of old currency networks, order and number must match with new addresses",
+    default="",
+    type=click.Path(dir_okay=False, writable=True),
+)
+@click.option(
+    "--new-addresses",
+    "new_addresses_file_path",
+    help="Path to a csv file with addresses of new currency networks, order and number must match with old addresses",
+    default="",
+    type=click.Path(dir_okay=False, writable=True),
+)
+@jsonrpc_option
+@gas_price_option
+@nonce_option
+@auto_nonce_option
+@keystore_option
+def migration(
+    old_addresses_file_path: str,
+    new_addresses_file_path: str,
+    jsonrpc: str,
+    gas_price: int,
+    nonce: int,
+    auto_nonce: bool,
+    keystore: str,
+):
+    """Used to migrate old currency networks to new ones
+    It will fetch information about users in the old contract and set them in the one
+    The address files should contain currency network addresses with
+    address matching from one file to the other from top to bottom"""
+
+    web3 = connect_to_json_rpc(jsonrpc)
+    private_key = retrieve_private_key(keystore)
+    nonce = get_nonce(
+        web3=web3, nonce=nonce, auto_nonce=auto_nonce, private_key=private_key
+    )
+    transaction_options = build_transaction_options(
+        gas=None, gas_price=gas_price, nonce=nonce
+    )
+    migrate_networks(
+        web3,
+        old_addresses_file_path,
+        new_addresses_file_path,
+        transaction_options,
+        private_key,
+    )
+
+
+@cli.command(short_help="Verify migration from old currency networks to new ones.")
+@click.option(
+    "--old-addresses",
+    "old_addresses_file_path",
+    help="Path to a csv file with addresses of old currency networks, order and number must match with new addresses",
+    default="",
+    type=click.Path(dir_okay=False, writable=True),
+)
+@click.option(
+    "--new-addresses",
+    "new_addresses_file_path",
+    help="Path to a csv file with addresses of new currency networks, order and number must match with old addresses",
+    default="",
+    type=click.Path(dir_okay=False, writable=True),
+)
+@jsonrpc_option
+def verify_migration(
+    old_addresses_file_path: str, new_addresses_file_path: str, jsonrpc: str
+):
+    """Used to verify migration of old currency networks to new ones
+    The address files should contain currency network addresses with
+    address matching from one file to the other from top to bottom"""
+
+    web3 = connect_to_json_rpc(jsonrpc)
+
+    verify_networks_migrations(web3, old_addresses_file_path, new_addresses_file_path)
+
+
+@cli.command(short_help="Deploy a new beacon contract")
+@click.option(
+    "--implementation",
+    "implementation_address",
+    help="Address of the implementation contract the beacon will point to",
+    required=True,
+    type=str,
+    callback=validate_address,
+)
+@click.option(
+    "--owner",
+    "owner_address",
+    help="Address of the owner of the beacon to be deployed",
+    required=True,
+    type=str,
+    callback=validate_address,
+)
+@jsonrpc_option
+@gas_price_option
+@nonce_option
+@auto_nonce_option
+@keystore_option
+def beacon(
+    implementation_address: str,
+    owner_address: str,
+    jsonrpc: str,
+    gas_price: int,
+    nonce: int,
+    auto_nonce: bool,
+    keystore: str,
+):
+    """Used to deploy an owned beacon pointing to an implementation address"""
+
+    web3 = connect_to_json_rpc(jsonrpc)
+    private_key = retrieve_private_key(keystore)
+    nonce = get_nonce(
+        web3=web3, nonce=nonce, auto_nonce=auto_nonce, private_key=private_key
+    )
+    transaction_options = build_transaction_options(
+        gas=None, gas_price=gas_price, nonce=nonce
+    )
+    beacon = deploy_beacon(
+        web3,
+        implementation_address,
+        owner_address,
+        private_key=private_key,
+        transaction_options=transaction_options,
+    )
+    click.secho(
+        f"Beacon successfully deployed at address {beacon.address} with owner {beacon.functions.owner().call()}"
+    )
+
+
+@cli.command(
+    short_help="Deploy new currrency networks and migrate old ones to deployed ones."
+)
+@click.option(
+    "--addresses-file",
+    "addresses_file_path",
+    help="Path to a csv file with addresses of old currency networks, order and number must match with new addresses",
+    default="",
+    type=click.Path(dir_okay=False, writable=True),
+)
+@click.option(
+    "--beacon",
+    "beacon_address",
+    help="Address of the beacon contract the proxies will point to",
+    required=True,
+    type=str,
+    callback=validate_address,
+)
+@click.option(
+    "--owner",
+    "owner_address",
+    help="Address of the owner of the proxies to be deployed",
+    required=True,
+    type=str,
+    callback=validate_address,
+)
+@jsonrpc_option
+@gas_price_option
+@nonce_option
+@auto_nonce_option
+@keystore_option
+def deploy_and_migrate(
+    addresses_file_path: str,
+    beacon_address: str,
+    owner_address: str,
+    jsonrpc: str,
+    gas_price: int,
+    nonce: int,
+    auto_nonce: bool,
+    keystore: str,
+):
+    web3 = connect_to_json_rpc(jsonrpc)
+    private_key = retrieve_private_key(keystore)
+    nonce = get_nonce(
+        web3=web3, nonce=nonce, auto_nonce=auto_nonce, private_key=private_key
+    )
+    transaction_options = build_transaction_options(
+        gas=None, gas_price=gas_price, nonce=nonce
+    )
+
+    deploy_and_migrate_networks_from_file(
+        web3=web3,
+        addresses_file_path=addresses_file_path,
+        beacon_address=beacon_address,
+        owner_address=owner_address,
+        private_key=private_key,
+        transaction_options=transaction_options,
+    )
